/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.model.listener;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import cz.nocach.swingmvp.MvpException;
import cz.nocach.swingmvp.util.Annotations;


/**
 * 
 * @author __nocach
 *
 */
public class CustomPropertyListenerInvokeInterface implements PropertyListenerInvokeInterface<Object> {
	private Map<String, Method> listenerMethodsForProperty = new HashMap<String, Method>();
	/**
	 * 
	 * @param listenerInterface valid custom model-interface
	 */
	public CustomPropertyListenerInvokeInterface(Class<?> listenerInterface) {
		for (Method each : listenerInterface.getDeclaredMethods()){
			Property propertyAnnotation = Annotations.getDeclaredAnnotationIfPresent(each, Property.class);
			if (propertyAnnotation == null) continue;
			each.setAccessible(true);
			listenerMethodsForProperty.put(propertyAnnotation.value(), each);
		}
	}
	@Override
	public void fire(Object listenerInstance, Object source, String propertyName,
			Object oldValue, Object newValue) {
		try {
			listenerMethodsForProperty.get(propertyName).invoke(listenerInstance, newValue);
		} catch (IllegalArgumentException e) {
			//TODO: Handle
			throw new MvpException(e);
		} catch (IllegalAccessException e) {
			//TODO: Handle
			throw new MvpException(e);
		} catch (InvocationTargetException e) {
			//TODO: Handle
			throw new MvpException(e);
		}
	}
	@Override
	public boolean supportsThisProperty(String propertyName) {
		return listenerMethodsForProperty.containsKey(propertyName);
	}
}
