/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.presenter;

import cz.nocach.swingmvp.model.Model;
/**
 * base class for creating Presenters.
 * @author __nocach
 *
 * @param <V> type of the view
 * @param <M> type of the model
 */
public class BasePresenter<V, M extends Model>{
	private V view;
	private M model;
	public BasePresenter(V view, M model) {
		this.view = view;
		this.model = model;
	}
	public V getView() {
		return view;
	}
	public void setView(V view) {
		this.view = view;
	}
	public M getModel() {
		return model;
	}
	public void setModel(M model) {
		this.model = model;
	}
	protected void autoSet(){
		throw new RuntimeException("not implemented");
		//will search annotation @Set("propertyName", auto=false) or @Set({"propertyOne", "propertyTwo"}, auto=false)
		//from method autoSet() was called
		//and will perform model.set${propertyName}()
		//for situations when client want's to have some code before setting propertyName
	}
	/**
	 * @param propertyName name of the models' property
	 * @param property value of the property to set
	 */
	void setModelProperty(String propertyName, Object property) {
		model.setProperty(propertyName, property);
	}
	/**
	 * 
	 * @param autoSet annotation from which models' propertyName can be extracted
	 * @param newValue value to be set to the propertyName from {@code autoSet}
	 */
	void setModelProperty(AutoSet autoSet, Object newValue) {
		setModelProperty(autoSet.value(), newValue);
	}
}
