/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.presenter;

import java.beans.PropertyChangeListener;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.google.common.base.Preconditions;
import com.jgoodies.binding.value.ValueModel;

import cz.nocach.swingmvp.util.Annotations;
/**
 * ValueModel that is interested only in {@link PresenterMethodSetValueModel#setValue(Object)}
 * other methods are simply ignored and return null if any result is expected
 * 
 * this {@code ValueModel} is used to call through reflection methods on the BasePresenter.
 * Call to {@code BasePresenter}s' method will not only invoke method, but also perform additional logic
 * that can not be done using aspects.
 * @author __nocach
 *
 */
public class PresenterMethodSetValueModel implements ValueModel{
	private final BasePresenter<?, ?> presenter;
	private final String method;
	private final Class<?> argType;

	public PresenterMethodSetValueModel(BasePresenter<?, ?> presenter, String method, Class<?> argType){
		this.argType = Preconditions.checkNotNull(argType);
		this.presenter = Preconditions.checkNotNull(presenter);
		this.method = Preconditions.checkNotNull(method);
	}

	@Override
	public Object getValue() {
		return null;
	}

	@Override
	public void setValue(Object newValue) {
		try {
			Method methodObject = presenter.getClass().getMethod(method, argType);
			AutoSet autoSet = Annotations.getDeclaredAnnotationIfPresent(methodObject, AutoSet.class);
			if (autoSet != null){
				presenter.setModelProperty(autoSet, newValue);
			}
			methodObject.setAccessible(true);
			methodObject.invoke(presenter, newValue);
		} catch (IllegalArgumentException e) {
			// TODO: handle
			throw new RuntimeException(e);
		} catch (SecurityException e) {
			// TODO: handle
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			// TODO: handle
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			// TODO: handle
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			// TODO: handle
			throw new RuntimeException(e);
		}
	}

	@Override
	public void addValueChangeListener(PropertyChangeListener listener) {
	}

	@Override
	public void removeValueChangeListener(PropertyChangeListener listener) {
	}

}
