/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.util;

import static com.google.common.base.Preconditions.checkNotNull;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author __nocach
 *
 */
public class Fields {
	public static Field getDeclaredFieldFromHierarchy(Class<?> clazz, String fieldName) throws NoSuchFieldException{
		checkNotNull(clazz);
		checkNotNull(fieldName);
		
		Class<?> currentClassInHierarchy = clazz;
		do {
			try{
				return currentClassInHierarchy.getDeclaredField(fieldName);
			}
			catch(NoSuchFieldException e){
				//continue searching
			}
		}
		//look up hierarchy until we have reached Object class
		while(!(currentClassInHierarchy = currentClassInHierarchy.getSuperclass())
				.equals(Object.class));
		
		throw new NoSuchFieldException(fieldName);
	}
	
	/**
	 * 
	 * @param fromChild not null 
	 * @param tillParent optional highest parent in hierarchy. If null then all hierarchy will
	 * 		be searched for declared fields. must be superclass of fromChild, or {@link IllegalArgumentException}
	 * 		will be thrown
	 * @return collection of declared fields for the hierarchy.
	 */
	public static Collection<Field> getDeclaredFieldsFromHierarchy(Class<?> fromChild, Class<?> tillParent){
		checkNotNull(fromChild);
		if (tillParent == null) tillParent = Object.class;
		if (!tillParent.isAssignableFrom(fromChild)){
			throw new IllegalArgumentException("tillParent "+tillParent+" must be above fromChild "
					+ fromChild + " in the hierarchy");
		}
		
		Map<String, Field> fieldNameAndField = new HashMap<String, Field>();
		Class<?> currentClassInHierarchy = fromChild;
		do {
			for (Field each : currentClassInHierarchy.getDeclaredFields()){
				//fields with same name make it into result only for the first 
				//match name
				if (fieldNameAndField.containsKey(each.getName())) continue;
				fieldNameAndField.put(each.getName(), each);
			}
		}
		//look up hierarchy until we have reached Object class
		while(!(currentClassInHierarchy = currentClassInHierarchy.getSuperclass())
				.equals(tillParent));
		return fieldNameAndField.values();
	}
	
}
