/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.model;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.fail;

import java.util.HashSet;
import java.util.Set;

import org.jmock.Mockery;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import cz.nocach.swingmvp.BaseTest;
import cz.nocach.swingmvp.model.listener.Listener;
import cz.nocach.swingmvp.testmodels.Listener1;
import cz.nocach.swingmvp.testmodels.Listener2;
import cz.nocach.swingmvp.testmodels.ListenerChild1;
import cz.nocach.swingmvp.testmodels.ListenerParent;
import cz.nocach.swingmvp.testmodels.ModelWithMultipleListeners;
import cz.nocach.swingmvp.util.FieldWithValue;

public class ModelListenerExtractorTest extends BaseTest{
	
	private ModelListenerExtractor listenerExtractor;
	private Set<FieldWithValue> resultFields;
	private ModelWithMultipleListeners model;
	
	@BeforeMethod 
	public void setUp(){
		listenerExtractor = new ModelListenerExtractor();
		model = new ModelWithMultipleListeners();
	}
	@Test
	public void testDiscoverAllClasses(){
		ModelWithMultipleListeners model = new ModelWithMultipleListeners();
		
		Set<Class<?>> possibleListenerTypes = listenerExtractor.getPossibleListenerClasses(model);
		
		assertTrue(possibleListenerTypes.contains(Listener1.class));
		assertTrue(possibleListenerTypes.contains(Listener2.class));
		assertTrue(possibleListenerTypes.contains(ListenerParent.class));
	}
	
	@Test
	public void testExtractingSingleListenerCollectionFields(){
		resultFields = listenerExtractor.getListenerFieldsWithValuesFor(model, ListenerParent.class);
		
		assertEquals(resultFields.size(), 1);
		assertFieldsHaveNotNullValue("listeners");
		assertFieldsHaveOfType("listeners", Set.class);
	}
	
	@Test
	public void testExtractingListenerCollectionAndSingleInstanceFields(){
		model.listener1 = mockery.mock(Listener1.class, "listener1Instance1");
		model.listenersListener1.add(mockery.mock(Listener1.class, "listener1instance2"));
		
		resultFields = listenerExtractor.getListenerFieldsWithValuesFor(model, Listener1.class);
		assertEquals(resultFields.size(), 3);
		assertFieldsHaveNotNullValue("listener1");
		assertFieldsHaveNullValue("listener1AnotherField");
		assertFieldsHaveOfType("listenersListener1", Set.class);
		assertFieldsHaveValueSetWithSize("listenersListener1", 1);
	}
	
	private void assertFieldsHaveOfType(String fieldName, Class type){
		for(FieldWithValue each : resultFields){
			if (each.field.getName().equals(fieldName)){
				assertTrue(type.isInstance(each.value));
				return;
			}
		}
		fail("result had no field named=" + fieldName);
	}
	
	private void assertFieldsHaveNullValue(String fieldName){
		for(FieldWithValue each : resultFields){
			if (each.field.getName().equals(fieldName)){
				assertNull(each.value);
				return;
			}
		}
		fail("result had no field named=" + fieldName);
	}
	private void assertFieldsHaveNotNullValue(String fieldName){
		for(FieldWithValue each : resultFields){
			if (each.field.getName().equals(fieldName)){
				assertNotNull(each.value);
				return;
			}
		}
		fail("result had no field named=" + fieldName);
	}
	private void assertFieldsHaveValueSetWithSize(String fieldName, int size){
		for(FieldWithValue each : resultFields){
			if (each.field.getName().equals(fieldName)){
				assertTrue(each.value instanceof Set);
				assertEquals(((Set)each.value).size(), size);
				return;
			}
		}
		fail("result had no field named=" + fieldName);
	}
}
