/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.model;

import java.util.Collection;
import java.util.Set;

import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import cz.nocach.swingmvp.model.listener.Add;
import cz.nocach.swingmvp.model.listener.Listener;
import cz.nocach.swingmvp.model.listener.ListenerException;
import cz.nocach.swingmvp.model.listener.ListenerTypeNotSpecified;
import cz.nocach.swingmvp.model.listener.Remove;
import cz.nocach.swingmvp.testobj.EmptyListener;


public class ModelValidatorTest {
	
	@Test(expectedExceptions=ListenerException.class, dataProvider="invalidCollectionListeners")
	public void throwIfModelListener_HasCollectionListener_WithInvalidCollectionProperty_OfAnnotation(
			Model model){
		ModelListenerValidator validator = new ModelListenerValidator(model);
		validator.checkListenerHasValidCollectionPropertyAnnotations();
	}
	@Test(expectedExceptions=ListenerTypeNotSpecified.class)
	public void throwIfModel_HasListenerCollection_WithoutListenerTypeSpecified(){
		ModelListenerValidator validator = new ModelListenerValidator(new ModelWithListenerCollectionWithoutListenerTypeSpecified());
		validator.checkCollectionOfListenersHasTypeSpecified();
	}
	@Test
	public void noThrowOnValidModelWithCollectionField(){
		ModelListenerValidator validator = new ModelListenerValidator(new ModelValidSimple());
		validator.checkListenerHasValidCollectionPropertyAnnotations();
		
		validator = new ModelListenerValidator(new ModelValidParent());
		validator.checkListenerHasValidCollectionPropertyAnnotations();
		
		validator = new ModelListenerValidator(new ModelValidChild());
		validator.checkListenerHasValidCollectionPropertyAnnotations();
	}
	public static class ModelValidSimple extends Model{
		@Listener
		private ModelValidListener listener;
		private Set<String> names;
	}
	public static class ModelValidParent extends Model{
		@Listener
		private ModelValidListener listener;
		private Set<String> names;
	}
	public static class ModelValidChild extends ModelValidParent{
	}
	public static interface ModelValidListener{
		@Add(collection="names", propertyName="name")
		public void onAddName(String name);
		@Remove(collection="names", propertyName="name")
		public void onRemoveName(String name);
	}
	public static class ModelWithInvalidCollectionListenersOne
			extends Model{
		@Listener
		private ListenerSpecifiedInvalidCollectionFieldNameInAdd listener;
	}
	public static class ModelWithInvalidCollectionListenersTwo
			extends Model{
		@Listener
		private ListenerSpecifiedInvalidCollectionFieldNameInRemove listener;
	}
	public static class ModelWithInvalidCollectionListenersThree
			extends Model{
		@Listener(type=ListenerSpecifiedInvalidCollectionFieldNameInAdd.class)
		private Set<ListenerSpecifiedInvalidCollectionFieldNameInAdd> listeners;
	}
	public static class ModelWithInvalidCollectionListenersFour
	extends Model{
		@Listener(type=ListenerSpecifiedInvalidCollectionFieldNameInRemove.class)
		private Set<ListenerSpecifiedInvalidCollectionFieldNameInRemove> listeners;
	}
	public static interface InvalidListener{};
	public static interface ListenerSpecifiedInvalidCollectionFieldNameInAdd extends InvalidListener{
		@Add(propertyName="propertyName", collection="notExists")
		public void onAdd(String value);
	}
	public static interface ListenerSpecifiedInvalidCollectionFieldNameInRemove extends InvalidListener{
		@Remove(propertyName="propertyName", collection="notExists")
		public void onRemove(String value);
	}
	public static class ModelWithListenerCollectionWithoutListenerTypeSpecified
			extends Model{
		@Listener
		private Collection<EmptyListener> listeners;
	}
	
	@DataProvider(name="invalidCollectionListeners")
	private Object[][] invalidCollectionListeners(){
		Object[][] result = new Object[4][1];
		result[0] = new Object[]{new ModelWithInvalidCollectionListenersOne()};
		result[1] = new Object[]{new ModelWithInvalidCollectionListenersTwo()};
		result[2] = new Object[]{new ModelWithInvalidCollectionListenersThree()};
		result[3] = new Object[]{new ModelWithInvalidCollectionListenersFour()};
		return result;
	}
}

