/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import cz.nocach.swingmvp.BaseTest;

public class ModelWithDefaultChangeListenerTest extends BaseTest {

	private DefaultPropertyChangeModel model;
	private TestModelPropertyChangeListener changeListener;

	@BeforeMethod
	public void setUp(){
		model = new DefaultPropertyChangeModel();
		changeListener = new TestModelPropertyChangeListener();
		model.addPropertyChangeListener(changeListener);
	}
	
	@Test
	public void testSettingSameValueTwiceDoesNotFireEvent() {
		model.setName("must fire event");
		assertTrue(changeListener.called);
		
		changeListener.called = false;
		model.setName("must fire event");
		assertFalse(changeListener.called, "setting same property fires only first time");
	}
	
	@Test
	public void testAutofireOfSimplePropertySetting() {
		model.name = "pre-set value";
		
		model.setName("must fire event");
		
		assertTrue(changeListener.called);
		assertEquals(model.getName(), "must fire event");
		assertEquals(changeListener.newValue, "must fire event");
		assertEquals(changeListener.oldValue, "pre-set value");
	}

	private class TestModelPropertyChangeListener implements PropertyChangeListener {
		private boolean called = false;
		private Object oldValue;
		private Object newValue;

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			called = true;
			oldValue = evt.getOldValue();
			newValue = evt.getNewValue();
		}
	}

	private static class DefaultPropertyChangeModel extends Model {
		private String name;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}
	}
}
