/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.model.listener;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import cz.nocach.swingmvp.BaseTest;
import cz.nocach.swingmvp.model.BeansPropertyChangeListenerInvokeInterface;
/**
 * tests {@link BeansPropertyChangeListenerInvokeInterface}
 * @author __nocach
 *
 */
public class CommonPropertyListenerInterfaceTest extends BaseTest{
	
	private Object source;
	private String propertyName;
	private Object oldValue;
	private String newValue;
	
	private BeansPropertyChangeListenerInvokeInterface beansPropertyInvoker;
	private CustomPropertyListenerInvokeInterface customListenerInvoker;

	@BeforeMethod
	public void setUp(){
		source = new Object();
		propertyName = "name";
		oldValue = null;
		newValue = "new value";
		beansPropertyInvoker = new BeansPropertyChangeListenerInvokeInterface();
		customListenerInvoker = new CustomPropertyListenerInvokeInterface(CustomListener.class);
	}
	
	
	@Test
	public void beansPropertyListenersSupportsEveryProperty(){
		assertTrue(beansPropertyInvoker.supportsThisProperty("randomProperty"));
	}
	
	@Test
	public void customListenerSupportsOnlyAnnotatedProperties(){
		assertTrue(customListenerInvoker.supportsThisProperty("name"));
		assertFalse(customListenerInvoker.supportsThisProperty("anotherName"));
	}
	
	@Test 
	public void customListenerInvoking(){
		CustomListenerStub customListenerStub = new CustomListenerStub();
		
		customListenerInvoker.fire(customListenerStub, source, propertyName, oldValue, newValue);
		
		assertEquals(customListenerStub.valueInOnNameChange, newValue);
	}
	
	@Test
	public void beansPropertyChangeListenerInvoking(){
		PropertyChangeListenerImplStub propertyChangeListenerImplStub = new PropertyChangeListenerImplStub();
		
		beansPropertyInvoker.fire(propertyChangeListenerImplStub, source, propertyName, oldValue, newValue);
		
		assertEquals(propertyChangeListenerImplStub.oldValueFromEvent, oldValue);
		assertEquals(propertyChangeListenerImplStub.newValueFromEvent, newValue);
		assertEquals(propertyChangeListenerImplStub.propertyNameFromEvent, propertyName);
		assertEquals(propertyChangeListenerImplStub.source, source);
	}
	
	public static class PropertyChangeListenerImplStub implements PropertyChangeListener{
		private String propertyNameFromEvent;
		private Object newValueFromEvent;
		private Object oldValueFromEvent;
		private Object source;
		@Override public void propertyChange(PropertyChangeEvent evt) {
			propertyNameFromEvent = evt.getPropertyName();
			newValueFromEvent = evt.getNewValue();
			oldValueFromEvent = evt.getOldValue();
			source = evt.getSource();
		}
	}
	
	public static interface CustomListener{
		@Property("name")
		public void onNameChange(String name);
	}
	
	public static class CustomListenerStub implements CustomListener{
		private String valueInOnNameChange;
		@Override public void onNameChange(String name) {
			valueInOnNameChange = name;
		}
	}
	
}
