/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.presenter;

import static org.testng.Assert.assertEquals;
import static org.testng.AssertJUnit.assertTrue;

import org.testng.annotations.Test;

import cz.nocach.swingmvp.autoinitiater.MvpAutoInitiater;
import cz.nocach.swingmvp.model.Model;
import cz.nocach.swingmvp.model.listener.Listener;
import cz.nocach.swingmvp.model.listener.Property;
import cz.nocach.swingmvp.testobj.EmptyView;
import cz.nocach.swingmvp.testobj.Person;
import cz.nocach.swingmvp.testobj.SimpleNameModel;
import cz.nocach.swingmvp.testobj.SimpleNameView;

public class PresenterDelegatorTest {
	@Test
	public void simpleAutoSetOnModel(){
		Person person = new Person();
		DelegatorTestPresenter presenter = new DelegatorTestPresenter(new SimpleNameView(), new SimpleNameModel(person));
		presenter.onNameChange("value to be auto-set on model");
		
		assertEquals(person.getName(), "value to be auto-set on model");
	}
	
	@Test
	public void testModelAutoSetIsPerformedBeforePresenterMethodCall(){
		SimpleNameModel model = new SimpleNameModel(new Person());
		model.setName("some start value");
		MvpAutoInitiater mvpInit = new MvpAutoInitiater(model, new SimpleNameView());
		PresenterToTestAutoSetInvocationOrder presenter = new PresenterToTestAutoSetInvocationOrder(new SimpleNameView(), model);
		
		presenter.onNameChange("new value");
		
		assertTrue(presenter.onNameChangeWasCalled);
		assertEquals(presenter.modelNameWhenEnteredIntoOnNameChange, "new value");
	}
	
	@Test
	public void testModelAutoSetWillAlsoFireAutomaticallyListenerEvents(){
		FiringTestModel model = new FiringTestModel();
		FiringTestModelListenerStub listener = new FiringTestModelListenerStub();
		model.listener = listener;
		FiringTestModelPresenter presenter = new FiringTestModelPresenter(new EmptyView(), model);
		String nameValue = "new name for auto-set";
		presenter.onNameChangeAutoSet(nameValue);
		
		assertEquals(model.getName(), nameValue);
		assertTrue(listener.onNameChangeCalled);
		assertEquals(listener.onNameChangeValue, nameValue);
		//TODO: add defaultPropertyChangeSupport test
		
		//TODO: add dead-lock test
	}
	
	public static class FiringTestModelPresenter extends BasePresenter<EmptyView, FiringTestModel>{
		public FiringTestModelPresenter(EmptyView view, FiringTestModel model) {
			super(view, model);
		}
		@AutoSet("name")
		public void onNameChangeAutoSet(String newName){
		}
	}
	public static class FiringTestModel extends Model{
		private String name;
		@Listener
		private TestModelFireModelListener listener;
		public String getName() {
			return name;
		}
		public void setName(String name) {
			this.name = name;
		}
	}
	public static class FiringTestModelListenerStub implements TestModelFireModelListener{
		public boolean onNameChangeCalled = false;
		public String onNameChangeValue = null;
		@Override
		public void onNameChange(String newName) {
			onNameChangeCalled = true;
			onNameChangeValue = newName;
		}
	}
	
	private static class DelegatorTestPresenter extends BasePresenter<SimpleNameView, SimpleNameModel>{
		public DelegatorTestPresenter(SimpleNameView view, SimpleNameModel model){
			super(view, model);
		}
		@AutoSet("name")
		public void onNameChange(String newValue){
			//empty?
			//automatically will call model.setName(newValue);
		}
	}
	
	public static interface TestModelFireModelListener{
		@Property("name")
		public void onNameChange(String newName);
	}
	
	public static class PresenterToTestAutoSetInvocationOrder extends BasePresenter<SimpleNameView, SimpleNameModel>{
		public PresenterToTestAutoSetInvocationOrder(SimpleNameView view, SimpleNameModel model) {
			super(view, model);
		}
		private boolean onNameChangeWasCalled;
		private String modelNameWhenEnteredIntoOnNameChange;

		@AutoSet("name")
		public void onNameChange(String newName){
			modelNameWhenEnteredIntoOnNameChange = getModel().getName();
			onNameChangeWasCalled = true;
		}
	}
}
