/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.util;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

import java.util.Collection;
import java.util.Set;

import org.testng.annotations.Test;

import com.google.common.collect.Sets;

import cz.nocach.swingmvp.testobj.ValueAnnotation;
import cz.nocach.swingmvp.testobj.ValueAnnotationAnother;

public class AnnotationsTest {
	@Test
	public void discoverAllMethodInterfaceAnnotations_ForAnnotationType(){
		Collection<ValueAnnotation> annotations = Annotations.getAnnotationsOnDeclaredMethods(
				InterfaceWithMethodsWithAnnotations.class, ValueAnnotation.class);
		
		assertAllMethodAnnotationDiscovered(annotations);
	}

	private void assertAllMethodAnnotationDiscovered(
			Collection<ValueAnnotation> annotations) {
		Set<String> expectedValues = Sets.newHashSet("", "two", "three");
		assertEquals(annotations.size(), expectedValues.size());
		for(ValueAnnotation each : annotations){
			boolean annotationWasInExpected = expectedValues.remove(each.value());
			assertTrue(annotationWasInExpected);
		}
		assertEquals(expectedValues.size(), 0, "not all annotations were discovered");
	}
	
	@Test
	public void discoverAllFieldAnnotations_ForAnnotationType(){
		Collection<ValueAnnotation> annotations = Annotations.getAnnotationsOnDeclaredFields(
				ClassWithFieldAnnotations.class, ValueAnnotation.class);
		
		assertAllFieldAnnotationDiscovered(annotations);
	}

	private void assertAllFieldAnnotationDiscovered(
			Collection<ValueAnnotation> annotations) {
		Set<String> expectedValues = Sets.newHashSet("", "two");
		assertEquals(annotations.size(), expectedValues.size());
		for(ValueAnnotation each : annotations){
			assertTrue(expectedValues.remove(each.value()), "annotation with value " + each.value() + " was not in expected values");
		}
		assertEquals(expectedValues.size(), 0, "not all annotations were discovered");
	}
	
	public static interface InterfaceWithMethodsWithAnnotations{
		@ValueAnnotation
		public void one();
		@ValueAnnotation("two")
		public void two();
		@ValueAnnotation("three")
		public void three();
	}
	
	public static class ClassWithFieldAnnotations{
		@ValueAnnotation
		private String one;
		@ValueAnnotation("two")
		private String two;
		private String noAnnotation;
		@ValueAnnotationAnother
		private String anotherAnnotation;
	}
}
