/*******************************************************************************
 * Copyright © 2011 Alexey Kaloshin. All Rights Reserved. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * 
 *  2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 
 *  3. The name of the author may not be used to endorse or promote 
 * products derived from this software without specific prior written permission. 
 * 
 *  THIS SOFTWARE IS PROVIDED BY Alexey Kaloshin "AS IS" AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package cz.nocach.swingmvp.util;

import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Set;

import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import com.google.common.collect.Sets;

import cz.nocach.swingmvp.BaseTest;

public class FieldsTest extends BaseTest{
	public static class Parent{
		private String privateParent;
		protected String protectedParent;
		public String publicParent;
		private String parendChildOverloaded;
		public String getParendChildOverloadedInParent() {
			return parendChildOverloaded;
		}
		public void setParendChildOverloadedInParent(String parendChildOverloaded) {
			this.parendChildOverloaded = parendChildOverloaded;
		}
		
	}
	public static class Child extends Parent{
		private String privateChild;
		protected String protectedChild;
		public String publicChild;
		private String parendChildOverloaded;
		public String getParendChildOverloadedInChild() {
			return parendChildOverloaded;
		}
		public void setParendChildOverloadedInChild(String parendChildOverloaded) {
			this.parendChildOverloaded = parendChildOverloaded;
		}
	}
	public static class ChildOfChild extends Child{
		private String privateChildChild;
	}
	@Test(dataProvider = "valid-fields")
	public void testDiscoverHierarchyField(Class<?> clazz, String fieldName) throws NoSuchFieldException{
		Field field = Fields.getDeclaredFieldFromHierarchy(clazz, fieldName);
		assertEquals(field.getName(), fieldName);
	}
	
	@Test(expectedExceptions=NoSuchFieldException.class)
	public void throwIfFieldDoesNotExist() throws NoSuchFieldException{
		Fields.getDeclaredFieldFromHierarchy(Child.class, "notExists");
	}
	
	public void getFieldFromClosestHierarchyClass() throws NoSuchFieldException, IllegalArgumentException, IllegalAccessException{
		Child child = new Child();
		child.setParendChildOverloadedInParent("parent");
		child.setParendChildOverloadedInChild("child");
		Field field = Fields.getDeclaredFieldFromHierarchy(child.getClass(), "parendChildOverloaded");
		field.setAccessible(true);
		String valuedInChild = (String) field.get(child);
		assertEquals(child.getParendChildOverloadedInChild(), valuedInChild);
	}
	
	@DataProvider(name="valid-fields")
	public Object[][] classAndFieldToSearch(){
		return new Object[][]{
			new Object[]{Child.class, "privateChild"}	
			, new Object[]{Child.class, "protectedChild"}	
			, new Object[]{Child.class, "publicChild"}	
			, new Object[]{Child.class, "publicParent"}	
			, new Object[]{Child.class, "protectedParent"}	
			, new Object[]{Child.class, "privateParent"}	
			, new Object[]{Parent.class, "privateParent"}	
			, new Object[]{Parent.class, "protectedParent"}	
			, new Object[]{Parent.class, "publicParent"}	
		};
	}
	
	public void getFieldsFromHierarchy(){
		Collection<Field> fields = Fields.getDeclaredFieldsFromHierarchy(ChildOfChild.class, Child.class);
		Set<String> expectedFieldNames = Sets.newHashSet("privateChildChild"
				, "parendChildOverloaded", "publicChild", "protectedChild", "privateChild");
		
		assertEquals(fields.size(), expectedFieldNames.size(), "size of discovered fields does not match expected size");
		for(Field each : fields){
			assertTrue(expectedFieldNames.remove(each.getName()), "expectedFieldNames did not contain " + each.getName());
		}
		assertTrue(expectedFieldNames.isEmpty(), "not all fields were discovered");
	}
}
